# -*- coding: utf-8 -*-
from django.utils.translation import gettext_lazy as _
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from thx.api.mixins import ThuxModelMixin
from thx.role.core.views import ThuxRoleViewSetMixin

from ... import models
from . import serializers, filters, permissions

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)


# OFFICE
class OfficeViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    OfficeViewSet
    -----------

    :action list: Return the office list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given office.
        `Profiles`: `superuser`, `staff`

    :action light: Return the office list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new office instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a office instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a office instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a office instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Office.objects.all()
    serializer_class = serializers.OfficeSerializer
    filter_backends = (filters.OfficeFilterBackend,)
    permission_classes = (permissions.OfficePermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a office instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a office instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# AREA
class AreaViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    AreaViewSet
    -----------

    :action list: Return the area list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given area.
        `Profiles`: `superuser`, `staff`

    :action light: Return the area list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new area instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a area instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a area instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a area instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Area.objects.all()
    serializer_class = serializers.AreaSerializer
    filter_backends = (filters.AreaFilterBackend,)
    permission_classes = (permissions.AreaPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a area instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a area instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)
